<?php
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Configuration;

class Bootstrap extends Zend_Application_Bootstrap_Bootstrap
{

    protected function _initFrontController() {
        
        $front = Zend_Controller_Front::getInstance();
        $access = new App_Access();
        $layout_manager = new App_Layout_Manager();

        $front->addModuleDirectory(APPLICATION_PATH . '/modules');

        $front->setDefaultModule('frontend');
        
        $front->registerPlugin($access);
        $front->registerPlugin($layout_manager);
        
        $front->setParam('prefixDefaultModule', true);

        
        return $front;

    }

    protected function _initRoutes()
    {
        //$this->addRoute('contato','frontend','contact','index');
        //$this->addRoute('JetSki Novos','frontend','product','find',array("mode" => "a"));
    }

    private function addRoute($expression,$module,$controller,$action,$extra = null)
    {
        $id = uniqid();
        $router = $this->frontController->getRouter();

        $params['module'] = $module;
        $params['controller'] = $controller;
        $params['action'] = $action;

        if(!is_null($extra))
        {
            foreach ($extra as $key => $value) {
                $params[$key] = $value;
            }
        }
        $route = new Zend_Controller_Router_Route($expression,$params);
        
        $router->addRoute($id,$route);
    }
    
    protected function _initApp()
    {
        //Armazena a sessão app do ini em memoria
        $app = $this->getOption('app');
        Zend_Registry::set("app", $app);
        ini_set("upload_max_filesize", "35M");
        
        //Libraries
        require_once "library/Resource.php";
        require_once "library/BreadCrumb.php";
        require_once "library/FormValidation.php";
        require_once "library/Permission.php";
        require_once "library/JQGrid.php";
        
    }

    protected function _initDoctype()
    {
        $this->bootstrap('view');
        $view = $this->getResource('view');
        $view->doctype('XHTML1_STRICT');
    }

    protected function _initDoctrine()
    {
        
        $options = $this->getOption("doctrine");
        //var_dump($options);
        //die();

        $config = new Configuration;
        $config->setProxyDir($options["proxy_dir"]);
        $config->setProxyNamespace($options["proxy_namespace"]);
        
        // AnnotationDriver
        $driver = $config->newDefaultAnnotationDriver($options["entities_dir"]);
        
        $config->setMetadataDriverImpl($driver);
        
        // Cache
        if($options["cache"]["enabled"])
        {
            $config->setMetadataCacheImpl(new $options["cache"]["driver"]);
            $config->setQueryCacheImpl(new $options["cache"]["driver"]);
            $config->setResultCacheImpl(new $options["cache"]["driver"]);
        }

        $config->setAutoGenerateProxyClasses($options["auto_generate_proxy"]);

        // SQL Log
        if($options["logger"]["enabled"])
        {
            $logger = new $options["logger"]["class"];
            $logger->fileName = $options["logger"]["file_name"];
            $config->setSQLLogger($logger);
        }

        $connectionOptions = array(
        'driver' => $options["connection"]["driver"],
        'user' => $options["connection"]["user"],
        'password' => $options["connection"]["password"],
        'dbname' => $options["connection"]["dbname"]
        );
        
        $em = EntityManager::create($connectionOptions, $config);
        
        Zend_Registry::set('em',$em);
        
        $this->em = $em;
        
        return $em;
        
    }

}

