<?php
namespace Entity;
use \DateTime;

/** @Entity
 *  @HasLifecycleCallbacks
 *  @Table(name="product")
 */
class Product
{

    /////////////////////////////////////////////////////////////////////////
    // Properties                                                          //
    /////////////////////////////////////////////////////////////////////////

    /**
    * @Id
    * @Column(type="integer")
    * @generatedValue(strategy="AUTO")
    */
    protected $id;
    /**
     * @Column(type="integer")
     */
    protected $measurement_unit;
    /**
     * @Column(type="integer")
     */
    protected $locale;    
    /**
     * @Column(length=100)
     */
    protected $name;    
    /**
     * @Column(length=100)
     */
    protected $code;
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $value=0;
    /**
     * @Column(type="decimal", precision=12, scale=3)
     */
    protected $thickness=0;
    /**
     * @Column(type="decimal", precision=12, scale=4)
     */
    protected $amount=0;    
    // Behaviors

    /**
     * @Column(type="datetime")
     */
    protected $created_at;
    /**
     * @Column(type="datetime")
     */
    protected $updated_at;
    /**
     * @Column(type="integer")
     * @version
     */
    protected $version;
    /**
     * @Column(type="integer")
     */
    protected $status;

    /////////////////////////////////////////////////////////////////////////
    // Relations                                                           //
    /////////////////////////////////////////////////////////////////////////
    
    /**
    * @OneToMany(targetEntity="History", mappedBy="product");
    */
    private $histories;
    
    /**
    * @OneToMany(targetEntity="Item", mappedBy="product");
    */
    private $itens;    
    
    /////////////////////////////////////////////////////////////////////////
    // Getters/Setters                                                     //
    /////////////////////////////////////////////////////////////////////////

    public function getId() {return $this->id;}
    public function setMeasurementUnit($value) {$this->measurement_unit = $value;}
    public function getMeasurementUnit() {return $this->measurement_unit;}
    public function setLocale($value) {$this->locale = $value;}
    public function getLocale() {return $this->locale;}
    public function setName($value) {$this->name = $value;}
    public function getName() {return $this->name;}
    public function setCode($value) {$this->code = $value;}
    public function getCode() {return $this->code;}
    public function setValue($value) {$this->value = str_replace(',','.',str_replace('.','',$value));}
    public function getValue() {return number_format($this->value, 2, ',', '.');}
    public function setThickness($value) {$this->thickness = str_replace(',','.',str_replace('.','',$value));}
    public function getThickness() {return number_format($this->thickness, 3, ',', '.');}
    public function setAmount($value) {$this->amount = str_replace(',','.',str_replace('.','',$value));}
    public function getAmount() {return number_format($this->amount, 4, ',', '.');}
    public function setStatus($value) {$this->status = $value;}
    public function getStatus() {return $this->status;}

    //Especiais
    public function popAmount($value)
    {
        $this->amount -= $value;
    }
    
    public function pushAmount($value)
    {
        $this->amount += $value;
    }    
    
    public function getCreatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getUpdatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getVersion() {return $this->version;}

    /////////////////////////////////////////////////////////////////////////
    // Constructor                                                         //
    /////////////////////////////////////////////////////////////////////////
    
    public function __construct()
    {
        $this->created_at = $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Hooks                                                               //
    /////////////////////////////////////////////////////////////////////////

    /**
     * @PrePersist
     * @PreUpdate
     */
    public function Validate()
    {
        $this->ApplyBehaviors();

        $e = new ValidationException();

        if(empty($this->code))
        {
            $e->AppendError("O Campo codigo não pode ser vazio.","code");
        }
        
        if(empty($this->name))
        {
            $e->AppendError("O Campo nome não pode ser vazio.","name");
        }

        if($e->hasError())
            throw $e;

    }

    private function ApplyBehaviors()
    {
        $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Helpers                                                             //
    /////////////////////////////////////////////////////////////////////////

    public function syncronize($data)
    {
        $this->setMeasurementUnit($data["measurement_unit"]);
        $this->setLocale($data["locale"]);
        $this->setName($data["name"]);
        $this->setCode($data["code"]);
        $this->setValue($data["value"]);
        $this->setThickness($data["thickness"]);
        $this->setAmount($data["amount"]);
        $this->setStatus($data["status"]);
    }

    public function toArray()
    {
        $data["id"] = $this->getId();
        $data["measurement_unit"] = $this->getMeasurementUnit();
        $data["locale"] = $this->getLocale();
        $data["name"] = $this->getName();
        $data["code"] = $this->getCode();
        $data["value"] = $this->getValue();
        $data["thickness"] = $this->getThickness();
        $data["amount"] = $this->getAmount();
        $data["status"] = $this->getStatus();

        return $data;
    }

}