<?php

class Frontend_SaleController extends Zend_Controller_Action
{

    public function  preDispatch()
    {
        $this->em = Zend_Registry::get("em");

        //desabilita o layout caso seja um XHR
        if ($this->_request->isXmlHttpRequest() || isset($_GET['ajax']))
            Zend_Controller_Action_HelperBroker::removeHelper('Layout');

        $this->ctrl = array(
            'name' => "sale",
            'class' => "Entity\Sale"
        );
        
        $this->view->location = "sale";
    }

    public function addAction()
    {
       BreadCrumb::clear();
       BreadCrumb::append("Vendas","/frontend/sale");
       BreadCrumb::append("Nova Venda","/frontend/sale/add");
    }
    
    public function viewAction()
    {
       BreadCrumb::clear();
       BreadCrumb::append("Vendas","/frontend/sale");
       BreadCrumb::append("Visualizar","/frontend/sale/view");
       
       $data = $this->_request->getParam("data");
       $id = $data["id"];
       
       $qb = $this->em->createQueryBuilder();
        
       $result = $qb->select("s,i,c,p,pa,u")
                ->from($this->ctrl["class"], "s")
                ->innerJoin("s.itens", "i")
                ->innerJoin("s.user", "u")
                ->innerJoin("i.product", "p")
                ->innerJoin("s.payments", "pa")
                ->innerJoin("s.customer", "c")
                ->where("s.id=".$id)
                ->getQuery()
                ->getArrayResult();    
       
       $this->view->sale = $result[0];
    }
    
    public function printAction()
    {
       Zend_Controller_Action_HelperBroker::removeHelper('Layout');
        
       BreadCrumb::clear();
       BreadCrumb::append("Vendas","/frontend/sale");
       BreadCrumb::append("Visualizar","/frontend/sale/view");
       
       $id = $this->_request->getParam("id");
       
       $qb = $this->em->createQueryBuilder();
        
       $result = $qb->select("s,i,c,p,pa,u ")
                ->from($this->ctrl["class"], "s")
                ->innerJoin("s.itens", "i")
                ->innerJoin("s.user", "u")
                ->innerJoin("i.product", "p")
                ->innerJoin("s.payments", "pa")
                ->innerJoin("s.customer", "c")
                ->where("s.id=".$id)
                ->getQuery()
                ->getArrayResult();    
       //var_dump($result[0]);
       $this->view->sale = $result[0];
       
    }
    
    public function toggleStatusAction()
    {
       $id = $this->_request->getParam("id");
       
       $sale = $this->em->find($this->ctrl["class"], $id);
       if($sale->getStatus()==1)
           $sale->setStatus(0);
       else
           $sale->setStatus(1);
       
       $this->em->persist($sale);
       $this->em->flush();
       $this->em->clear();
       
       $this->_redirect("frontend/sale/view?data[id]=" .$id);
       
    }
    
    public function finalizeAction()
    {
        $data = $this->_request->getParam("data");
        //var_dump($data);
        
        //Vefificação
        if(empty($data["payment"]))
        {
            $response["result"]="err";        
            $response["message"]="Não é possível realizar uma compra sem pagamento!";        
            die(json_encode($response));
        }
        
        //Salva Compra
        $s = new Entity\Sale();
        $s->syncronize($data);
        $s->setUserId(Resource::getCurrentUserId());
        $s->setReturnedTotal(0);
        $s->setStatus(0);
        $this->em->persist($s);
        $this->em->flush();
        $this->em->clear();

        file_put_contents("sale.log",$s->getId() . "\n\r",FILE_APPEND);
        file_put_contents("sale.log",print_r($data,true),FILE_APPEND);
        
        //Salva os pagamentos
        foreach($data["payment"] as $payment)
        {            
            $payment_data = json_decode($payment,true);
            //var_dump($data["payment"]);
            //die();
            $p = new Entity\Payment();
            $p->syncronize($payment_data);
            $p->setSaleId($s->getId());
            $this->em->persist($p);
            $this->em->flush();
            $this->em->clear();
        }
        
        //Salva Items
        foreach($data["item"] as $item)
        {            
            $item_data = json_decode($item,true);
            
            $i = new Entity\Item();
            $i->syncronize($item_data);
            $i->setSaleId($s->getId());
            $i->setReturnedAmount(0);
            $this->em->persist($i);

            $this->em->flush();
            $this->em->clear();
        
            //Gera histórico            
            $h = new Entity\History();
            $h->setOperation(Resource::OPERATION_SALE);
            $h->setSaleId($s->getId());
            $h->setProductId($item_data["product_id"]);
            $h->setUserId(Resource::getCurrentUserId());
            $h->setValue($item_data["value"]);
            $h->setTotal($item_data["sub_total"]);
            $h->setAmount($item_data["volume"]);
            
            $this->em->persist($h);
            
            $this->em->flush();
            $this->em->clear();
            
            //Tira do estoque
            Resource::stockPop($item_data["product_id"],str_replace(',','.',str_replace('.','',$item_data["volume"])));            
        }
        
        $this->em->flush();
        $this->em->clear();
        
        $response["result"]="ok";        
        $response["sale_id"]=$s->getId();;        
        die(json_encode($response));
    }
    
    public function execReturnAction()
    {
        
        $item_id = $this->_request->getParam("item_id");
        $amount = $this->_request->getParam("amount");// Aqui vem em quantidade (não é volume)
        
        $item = $this->em->find("Entity\Item", $item_id);
        
        if($amount > ($item->getAmount() - $item->getReturnedAmount()))
        {
            die("Quantidade máxima para devolução é: " . ($item->getAmount() - $item->getReturnedAmount()));
        }
        
        $item->addReturnedAmount($amount);
        
        $this->em->persist($item);
        $this->em->flush();
        $this->em->clear();
        
        //Atualiza estoque (Produto)
        $product = $item->getProduct();
        
        $returned_amount = 0;
        if($product->getMeasurementUnit()==0)//Peça
            $returned_amount = $amount;
        else
            $returned_amount = (($item->getVolumeEx() * $amount) / $item->getAmount());
        
        $product->pushAmount($returned_amount);

        $this->em->persist($product);
        $this->em->flush();
        $this->em->clear();
        
        //Caucula valor a ser devolvido
        $returned_value = $returned_amount * $item->getValueEx();
        
        //Atualiza valor devolvido na venda        
        $sale = $item->getSale();
        
        $sale->addToReturnedTotal($returned_value);
        
        $this->em->persist($sale);
        $this->em->flush();
        $this->em->clear();
        
        //Gera histórico            
        $h = new Entity\History();
        $h->setOperation(Resource::OPERATION_ENTRANCE_ADJUST);
        $h->setSaleId($sale->getId());
        $h->setProductId($product->getId());
        $h->setUserId(Resource::getCurrentUserId());
        $h->setValue($item->getValueEx());//Utilizado só mente na compra
        $h->setTotal(number_format($returned_value,2,",","."));
        $h->setAmount(number_format($returned_amount,4,",","."));

        $this->em->persist($h);

        $this->em->flush();
        $this->em->clear();

        
        die("ok");
    }
    
    public function indexAction()
    {
       BreadCrumb::clear();
       BreadCrumb::append("Vendas - Histórico","/frontend/sale");
    }
    
    public function listAction()
    {
        $qb = $this->em->createQueryBuilder();
        
        $date_start = $this->_request->getParam("date_start");
        $date_end = $this->_request->getParam("date_end");
        $export = $this->_request->getParam("export");
        
        $qb->from($this->ctrl["class"], "s")
           ->innerJoin("s.customer","c")
           ->innerJoin("s.user", "u");
                
        if(!empty($date_start))
        {
            $tmp = explode("/",$date_start);
            $date_start = $tmp[2] . "-" . $tmp[1] . "-" . $tmp[0] . " 00:00:00"; 
            $qb->andWhere("s.created_at >= '$date_start'");
        }
        
        if(!empty($date_end))
        {
            $tmp = explode("/",$date_end);
            $date_end = $tmp[2] . "-" . $tmp[1] . "-" . $tmp[0] . " 23:59:59"; 
            $qb->andWhere("s.created_at <= '$date_end'");
        }             

        $select = "s.id,u.name as u_name,s.status,(select p.pay_mode from Entity\Payment p where p.sale_id=s.id GROUP BY p.sale_id) as payment,s.volume,c.name,s.created_at,s.delivery,s.interest,s.service,s.discount,s.others,s.total,s.final_total,(s.final_total - s.returned_total) as total_geral";
        
        $filters = "s.id,u.name,payment,s.volume,c.name,s.created_at,s.delivery,s.interest,s.service,s.others,s.total,s.final_total,total_geral";
        $filters = explode(",",$filters);
        
        $data = JQGrid::getJson($qb, $this->_request, $select, $filters, "s",false,array("volume","final_total"));
        
        $data = json_decode($data,true);

        $sum = JQGrid::getSum($qb, $this->_request,"sum(s.final_total) as sum_final_total,sum(s.volume) as sum_volume,sum(s.discount) as sum_discount,sum(s.returned_total) as sum_returned_total", $filters, "s");
        
        $data["userdata"]['volume'] = $sum[0]["sum_volume"];
        $data["userdata"]['final_total'] = $sum[0]["sum_final_total"];
        $data["userdata"]['discount'] = $sum[0]["sum_discount"];
        $data["userdata"]['total_geral'] = $sum[0]["sum_final_total"] - $sum[0]["sum_returned_total"];

        $data = json_encode($data);
        
        if($export)
        {
            $this->view->data = json_decode($data,true);
            $this->render("export");
        }
        
        die($data);
    }
    
    public function permissionDeniedAction()
    {
       BreadCrumb::clear();
       BreadCrumb::append("Home","/frontend/index/");
       BreadCrumb::append("Permissão negada","/frontend/index/permission-denied");
    }

}
