<?php
namespace Entity;
use \DateTime;

/** @Entity
 *  @HasLifecycleCallbacks
 *  @Table(name="customer")
 */
class Customer
{

    
    /////////////////////////////////////////////////////////////////////////
    // Properties                                                          //
    /////////////////////////////////////////////////////////////////////////

    /**
    * @Id
    * @Column(type="integer")
    * @generatedValue(strategy="AUTO")
    */
    protected $id;
    /**
     * @Column(type="integer")
     */
    protected $type;
    /**
     * @Column(length=100)
     */
    protected $name;    
    /**
     * @Column(length=100)
     */
    protected $nick_name;    
    /**
     * @Column(length=100)
     */
    protected $address;
    /**
     * @Column(length=100)
     */
    protected $complement;
    /**
     * @Column(length=50)
     */
    protected $neighborhood;
    /**
     * @Column(length=50)
     */
    protected $city;
    /**
     * @Column(length=2)
     */
    protected $uf;
    /**
     * @Column(length=15)
     */
    protected $zipcode;    
    /**
     * @Column(length=20)
     */
    protected $phone;    
    /**
     * @Column(length=100)
     */
    protected $email;    
    /**
     * @Column(length=20)
     */
    protected $cellphone;    
    /**
     * @Column(length=20)
     */
    protected $cpf;    
    /**
     * @Column(length=20)
     */
    protected $cnpj;    
    /**
     * @Column(length=20)
     */
    protected $rg;    
    /**
     * @Column(length=20)
     */
    protected $ie;    
    /**
     * @Column(type="integer")
     */
    protected $status;    
    /**
     * @Column(length=255)
     */
    protected $obs;
    // Behaviors

    /**
     * @Column(type="datetime")
     */
    protected $created_at;
    /**
     * @Column(type="datetime")
     */
    protected $updated_at;
    /**
     * @Column(type="integer")
     * @version
     */
    protected $version;

    /////////////////////////////////////////////////////////////////////////
    // Relations                                                           //
    /////////////////////////////////////////////////////////////////////////

    /**
    * @OneToMany(targetEntity="Sale", mappedBy="customer");
    */
    private $sales;
    
    /////////////////////////////////////////////////////////////////////////
    // Getters/Setters                                                     //
    /////////////////////////////////////////////////////////////////////////

    public function getId() {return $this->id;}
    public function setType($value) {$this->type = $value;}
    public function getType() {return $this->type;}
    public function setName($value) {$this->name = $value;}
    public function getName() {return $this->name;}
    public function setNickName($value) {$this->nick_name = $value;}
    public function getNickName() {return $this->nick_name;}
    public function setAddress($value) {$this->address = $value;}
    public function getAddress() {return $this->address;}
    public function setComplement($value) {$this->complement = $value;}
    public function getComplement() {return $this->complement;}
    public function setNeighborhood($value) {$this->neighborhood = $value;}
    public function getNeighborhood() {return $this->neighborhood;}
    public function setCity($value) {$this->city = $value;}
    public function getCity() {return $this->city;}
    public function setUF($value) {$this->uf = $value;}
    public function getUF() {return $this->uf;}
    public function setZipcode($value) {$this->zipcode = $value;}
    public function getZipcode() {return $this->zipcode;}
    public function setPhone($value) {$this->phone = $value;}
    public function getPhone() {return $this->phone;}
    public function setEmail($value) {$this->email = $value;}
    public function getEmail() {return $this->email;}
    public function setCellPhone($value) {$this->cellphone = $value;}
    public function getCellPhone() {return $this->cellphone;}
    public function setCpf($value) {$this->cpf = $value;}
    public function getCpf() {return $this->cpf;}
    public function setCnpj($value) {$this->cnpj = $value;}
    public function getCnpj() {return $this->cnpj;}
    public function setRg($value) {$this->rg = $value;}
    public function getRg() {return $this->rg;}
    public function setIe($value) {$this->ie = $value;}
    public function getIe() {return $this->ie;}
    public function setStatus($value) {$this->status = $value;}
    public function getStatus() {return $this->status;}
    public function setObs($value) {$this->obs = $value;}
    public function getObs() {return $this->obs;}

    public function getCreatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getUpdatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getVersion() {return $this->version;}

    /////////////////////////////////////////////////////////////////////////
    // Constructor                                                         //
    /////////////////////////////////////////////////////////////////////////
    
    public function __construct()
    {
        $this->created_at = $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Hooks                                                               //
    /////////////////////////////////////////////////////////////////////////

    /**
     * @PrePersist
     * @PreUpdate
     */
    public function Validate()
    {
        $this->ApplyBehaviors();

        $e = new ValidationException();

        if(empty($this->name))
        {
            $e->AppendError("O Campo nome não pode ser vazio.","name");
        }

        if(empty($this->phone))
        {
            $e->AppendError("Telefone obrigatório","phone");
        }
        
        if($e->hasError())
            throw $e;

    }

    private function ApplyBehaviors()
    {
        $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Helpers                                                             //
    /////////////////////////////////////////////////////////////////////////

    public function syncronize($data)
    {
        $this->setType($data["type"]);
        $this->setName($data["name"]);
        $this->setNickName($data["nick_name"]);
        $this->setAddress($data["address"]);
        $this->setComplement($data["complement"]);
        $this->setNeighborhood($data["neighborhood"]);
        $this->setCity($data["city"]);
        $this->setUF($data["uf"]);
        $this->setZipcode($data["zipcode"]);
        $this->setPhone($data["phone"]);
        $this->setEmail($data["email"]);
        $this->setCellPhone($data["cellphone"]);
        
        $this->setCpf($data["cpf"]);
        $this->setCnpj($data["cnpj"]);
        $this->setRg($data["rg"]);
        $this->setIe($data["ie"]);
        
        $this->setStatus($data["status"]);
        $this->setObs($data["obs"]);
        
    }

    public function toArray()
    {
        $data["id"] = $this->getId();
        $data["type"] = $this->getType();
        $data["name"] = $this->getName();
        $data["nick_name"] = $this->getNickName();
        $data["address"] = $this->getAddress();
        $data["complement"] = $this->getComplement();
        $data["neighborhood"] = $this->getNeighborhood();
        $data["city"] = $this->getCity();
        $data["uf"] = $this->getUF();
        $data["zipcode"] = $this->getZipcode();
        $data["phone"] = $this->getPhone();
        $data["email"] = $this->getEmail();
        $data["cellphone"] = $this->getCellPhone();
        $data["cpf"] = $this->getCpf();
        $data["cnpj"] = $this->getCnpj();
        $data["rg"] = $this->getRg();
        $data["ie"] = $this->getIe();
        $data["status"] = $this->getStatus();
        $data["obs"] = $this->getObs();

        return $data;
    }

}