<?php
namespace Entity;
use \DateTime;

/** @Entity
 *  @HasLifecycleCallbacks
 *  @Table(name="sale")
 */
class Sale
{

    /////////////////////////////////////////////////////////////////////////
    // Properties                                                          //
    /////////////////////////////////////////////////////////////////////////

    /**
    * @Id
    * @Column(type="integer")
    * @generatedValue(strategy="AUTO")
    */
    protected $id;
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $delivery=0;
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $interest=0;
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $service=0;
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $discount=0;    
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $others=0;
    /**
     * @Column(type="decimal", precision=12, scale=4)
     */
    protected $volume=0;     
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $total=0;    
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $final_total=0;
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $returned_total=0;
    /**
     * @Column(type="integer")
     */
    protected $status;    
    
    // Behaviors

    /**
     * @Column(type="datetime")
     */
    protected $created_at;
    /**
     * @Column(type="datetime")
     */
    protected $updated_at;
    /**
     * @Column(type="integer")
     * @version
     */
    protected $version;

    /////////////////////////////////////////////////////////////////////////
    // Relations                                                           //
    /////////////////////////////////////////////////////////////////////////
    
    /**
    * @ManyToOne(targetEntity="Customer", inversedBy="sales")
    * @JoinColumn(name="customer_id", referencedColumnName="id")
    */
    private $customer;
    
    /**
    * @ManyToOne(targetEntity="User", inversedBy="sales")
    * @JoinColumn(name="user_id", referencedColumnName="id")
    */
    private $user;
    
    /**
    * @OneToMany(targetEntity="Item", mappedBy="sale");
    */
    private $itens;
    
    /**
    * @OneToMany(targetEntity="Payment", mappedBy="sale");
    */
    private $payments;
    
    /**
    * @OneToMany(targetEntity="History", mappedBy="sale");
    */
    private $histories;    
    
    /////////////////////////////////////////////////////////////////////////
    // ForeingKey                                                          //
    /////////////////////////////////////////////////////////////////////////

    /**
     * @Column(type="integer")
     */
    protected $customer_id; 
    
    /**
     * @Column(type="integer")
     */
    protected $user_id;     
    
    /////////////////////////////////////////////////////////////////////////
    // Getters/Setters                                                     //
    /////////////////////////////////////////////////////////////////////////

    public function getId() {return $this->id;}
    public function setDelivery($value) {$this->delivery = str_replace(',','.',str_replace('.','',$value));}
    public function getDelivery() {return number_format($this->delivery, 2, ',', '.');}
    public function setInterest($value) {$this->interest = str_replace(',','.',str_replace('.','',$value));}
    public function getInterest() {return number_format($this->interest, 2, ',', '.');}
    public function setService($value) {$this->service = str_replace(',','.',str_replace('.','',$value));}
    public function getService() {return number_format($this->service, 2, ',', '.');}
    public function setOthers($value) {$this->others = str_replace(',','.',str_replace('.','',$value));}
    public function getOthers() {return number_format($this->others, 2, ',', '.');}
    public function setDiscount($value) {$this->discount = str_replace(',','.',str_replace('.','',$value));}
    public function getDiscount() {return number_format($this->discount, 2, ',', '.');}
    public function setVolume($value) {$this->volume = str_replace(',','.',str_replace('.','',$value));}
    public function getVolume() {return number_format($this->volume, 4, ',', '.');}
    public function setTotal($value) {$this->total = str_replace(',','.',str_replace('.','',$value));}
    public function getTotal() {return number_format($this->total, 2, ',', '.');}
    public function setFinalTotal($value) {$this->final_total = str_replace(',','.',str_replace('.','',$value));}
    public function getFinalTotal() {return number_format($this->final_total, 2, ',', '.');}
    public function setReturnedTotal($value) {$this->returned_total = str_replace(',','.',str_replace('.','',$value));}
    public function getReturnedTotal() {return number_format($this->returned_total, 2, ',', '.');}
    public function setStatus($value) {$this->status = $value;}
    public function getStatus() {return $this->status;}
    
    //Especial
    public function addToReturnedTotal($value) {$this->returned_total += $value;}
    
    public function setCustomer($value) {$this->customer = $value;}
    public function getCustomer() {return $this->customer;}
    public function setCustomerId($value) {$this->customer_id = $value;}
    public function getCustomerId() {return $this->customer_id;}    

    public function setUser($value) {$this->user = $value;}
    public function getUser() {return $this->user;}
    public function setUserId($value) {$this->user_id = $value;}
    public function getUserId() {return $this->user_id;}    
    
    public function getCreatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getUpdatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getVersion() {return $this->version;}

    /////////////////////////////////////////////////////////////////////////
    // Constructor                                                         //
    /////////////////////////////////////////////////////////////////////////
    
    public function __construct()
    {
        $this->created_at = $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Hooks                                                               //
    /////////////////////////////////////////////////////////////////////////

    /**
     * @PrePersist
     * @PreUpdate
     */
    public function Validate()
    {
        $this->ApplyBehaviors();

        $e = new ValidationException();

        if($e->hasError())
            throw $e;

    }

    private function ApplyBehaviors()
    {
        $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Helpers                                                             //
    /////////////////////////////////////////////////////////////////////////

    public function syncronize($data)
    {
        $this->setDelivery($data["delivery"]);
        $this->setInterest($data["interest"]);
        $this->setService($data["service"]);
        $this->setOthers($data["others"]);
        $this->setDiscount($data["discount"]);
        $this->setVolume($data["volume"]);
        $this->setTotal($data["total"]);
        $this->setFinalTotal($data["final_total"]);
        $this->setReturnedTotal($data["returned_total"]);
        $this->setStatus($data["status"]);
        
        $this->setCustomerId($data["customer_id"]);        
        $this->setUserId($data["user_id"]);        
    }

    public function toArray()
    {
        $data["id"] = $this->getId();
        $data["delivery"] = $this->getLocale();
        $data["interest"] = $this->getName();
        $data["others"] = $this->getCode();
        $data["volume"] = $this->getVolume();
        $data["total"] = $this->getTotal();
        $data["discount"] = $this->getDiscount();
        $data["final_total"] = $this->getFinalTotal();
        $data["returned_total"] = $this->getReturnedTotal();
        $data["status"] = $this->getStatus();
        
        $this->getCustomerId($data["customer_id"]);
        $this->getUserId($data["user_id"]);

        return $data;
    }

}