<?php

class Frontend_CustomerController extends Zend_Controller_Action {

    
    public function preDispatch()
    {
        $this->em = Zend_Registry::get("em");

        //desabilita o layout caso seja um XHR/Ajax
        if ($this->_request->isXmlHttpRequest() || isset($_GET['ajax']))
            Zend_Controller_Action_HelperBroker::removeHelper('Layout');

        $this->ctrl = array(
            'name' => "customer",
            'class' => "Entity\Customer"
        );
        
        $this->view->location = "customer";
    }

    public function indexAction()
    {
        BreadCrumb::clear();
        BreadCrumb::append("Clientes","/frontend/customer");
        
       if(!Permission::isChecked(Permission::VIEW_CUSTOMER,Resource::getCurrentUserPermissionData()))
       {
           $this->_redirect("/frontend/index/permission-denied");
       }
        
    }

    public function listAction()
    {
        $qb = $this->em->createQueryBuilder();

        $qb->from($this->ctrl["class"], "c");

        $select = "c.id,c.name,c.neighborhood,c.address,c.city,c.phone,c.status,c.cpf,c.cnpj,c.nick_name";

        $filters = explode(",",$select);
        
        die(JQGrid::getJson($qb, $this->_request, $select, $filters, "c",false));
    }

    public function getInfoAction()
    {
        
        $id = $this->_request->getParam("id");
        $obj = $this->em->find($this->ctrl["class"], $id);
        
        $this->view->customer = $obj->toArray();
    }
    
    public function addAction()
    {
        
        if(!Permission::isChecked(Permission::ADD_EDIT_CUSTOMER,Resource::getCurrentUserPermissionData()))
        {
           $this->_redirect("/frontend/index/permission-denied");
        }
        
        BreadCrumb::clear();
        BreadCrumb::append("Clientes","/frontend/customer");
        
        
        $data = $this->_request->getParam("data");
        $id = $data["id"];
        
        if(empty($id))
            BreadCrumb::append("Adicionar","/frontend/customer/add");
        else
            BreadCrumb::append("Editar","/frontend/customer/add");

        if ($this->_request->isPost()) {
            
            if (empty($id))
                $obj = new $this->ctrl["class"]();
            else
                $obj = $this->em->find($this->ctrl["class"], $id);

            $obj->syncronize($data);

            try {
                $this->em->persist($obj);
                $this->em->flush();
                $this->em->clear();
                $obj = $this->em->find($this->ctrl["class"], $obj->getId());
                $this->view->data = $obj->toArray();
                $this->view->show_success = true;
            } catch (\Entity\ValidationException $e) {
                $this->view->errors = $e->getMessages();
                $this->view->data = $data;
            } catch (Exception $e) {
                die("Erro Fatal: " . $e->getMessage());
            }
        } else {
            if (!is_null($id)) {
                $obj = $this->em->find($this->ctrl["class"], $id);
                $this->view->data = $obj->toArray();
            }
        }

        //Resources
    }

    public function deleteAction() {

        if(!Permission::isChecked(Permission::DELETE_USER,Resource::getCurrentUserPermissionData()))
        {
           die("permission-denied");
        }
        
        try
        {
            $id = $this->_request->getParam("id");
            $obj = $this->em->find($this->ctrl["class"], $id);

            $this->em->remove($obj);
            $this->em->flush();

            die("ok");
        }
        catch(Exception $e)
        {
            die("err");
        }
    }
    
    public function searchAction()
    {
        
        $term = $this->_request->getParam("term");
        $qb = $this->em->createQueryBuilder();
        
        $qb->select("c.id,c.name,c.nick_name,c.phone")
           ->from($this->ctrl["class"], "c")
            ->where('c.status=0')
           ->setMaxResults(10);
                   
        $w1 = $qb->expr()->like("c.name","'%" . $term ."%'");
        $w2 = $qb->expr()->like("c.nick_name","'%" . $term ."%'");
        
        $qb->andWhere($qb->expr()->orx($w1,$w2));    
        
        $result = $qb->getQuery()->getArrayResult();

        $data = array();
        
        foreach($result as $c)
        {
            $data[] = array("id"=>$c["id"],"label"=>$c["name"] . " - " . $c["nick_name"]);
        }
        
        die(json_encode($data));
        
    }

}
