<?php
namespace Entity;
use \DateTime;

/** @Entity
 *  @HasLifecycleCallbacks
 *  @Table(name="history")
 */
class History
{

    /////////////////////////////////////////////////////////////////////////
    // Properties                                                          //
    /////////////////////////////////////////////////////////////////////////

    /**
    * @Id
    * @Column(type="integer")
    * @generatedValue(strategy="AUTO")
    */
    protected $id;
    /**
     * @Column(type="integer")
     */
    protected $operation;
    /**
     * @Column(type="decimal", precision=12, scale=4)
     */
    protected $amount=0;
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $value=0;
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $total=0;    
    // Behaviors

    /**
     * @Column(type="datetime")
     */
    protected $created_at;
    /**
     * @Column(type="datetime")
     */
    protected $updated_at;
    /**
     * @Column(type="integer")
     * @version
     */
    protected $version;

    /////////////////////////////////////////////////////////////////////////
    // Relations                                                           //
    /////////////////////////////////////////////////////////////////////////

    /**
    * @ManyToOne(targetEntity="User", inversedBy="histories")
    * @JoinColumn(name="user_id", referencedColumnName="id")
    */
    private $user;    
    
    /**
    * @ManyToOne(targetEntity="Product", inversedBy="histories")
    * @JoinColumn(name="product_id", referencedColumnName="id")
    */
    private $product;
    
    /**
    * @ManyToOne(targetEntity="Sale", inversedBy="histories")
    * @JoinColumn(name="sale_id", referencedColumnName="id")
    */
    private $sale;    
    
    /////////////////////////////////////////////////////////////////////////
    // ForeingKey                                                           //
    /////////////////////////////////////////////////////////////////////////

    /**
     * @Column(type="integer")
     */
    protected $user_id;
    /**
     * @Column(type="integer")
     */
    protected $product_id;    
    /**
     * @Column(type="integer",nullable="true")
     */
    protected $sale_id;    
    
    /////////////////////////////////////////////////////////////////////////
    // Getters/Setters                                                     //
    /////////////////////////////////////////////////////////////////////////

    public function getId() {return $this->id;}
    public function setOperation($value) {$this->operation = $value;}
    public function getOperation() {return $this->operation;}
    public function setAmount($value) {$this->amount = str_replace(',','.',str_replace('.','',$value));}
    public function getAmount() {return number_format($this->amount, 4, ',', '.');}
    public function setValue($value) {$this->value = str_replace(',','.',str_replace('.','',$value));}
    public function getValue() {return number_format($this->value, 2, ',', '.');}
    public function setTotal($value) {$this->total = str_replace(',','.',str_replace('.','',$value));}
    public function getTotal() {return number_format($this->total, 2, ',', '.');}

    //Especiais 
    public function setAmountEx($value) {$this->amount = $value;}
    public function setValueEx($value) {$this->value = $value;}
    public function setTotalEx($value) {$this->total = $value;}
    
    
    public function setUser($value) {$this->user = $value;}
    public function getUser() {return $this->user;}
    public function setUserId($value) {$this->user_id = $value;}
    public function getUserId() {return $this->user_id;}
    
    public function setProduct($value) {$this->product = $value;}
    public function getProduct() {return $this->product;}
    public function setProductId($value) {$this->product_id = $value;}
    public function getProductId() {return $this->product_id;}

    public function setSale($value) {$this->sale = $value;}
    public function getSale() {return $this->sale;}
    public function setSaleId($value) {$this->sale_id = $value;}
    public function getSaleId() {return $this->sale_id;}
    
    public function getCreatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getUpdatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getVersion() {return $this->version;}

    /////////////////////////////////////////////////////////////////////////
    // Constructor                                                         //
    /////////////////////////////////////////////////////////////////////////
    
    public function __construct()
    {
        $this->created_at = $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Hooks                                                               //
    /////////////////////////////////////////////////////////////////////////

    /**
     * @PrePersist
     * @PreUpdate
     */
    public function Validate()
    {
        $this->ApplyBehaviors();

        $e = new ValidationException();

        if($e->hasError())
            throw $e;

    }

    private function ApplyBehaviors()
    {
        $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Helpers                                                             //
    /////////////////////////////////////////////////////////////////////////

    public function syncronize($data)
    {
        $this->setOperation($data["operation"]);
        $this->setAmount($data["amount"]);
        $this->setTotal($data["total"]);
        $this->value = $this->total / $this->amount;
        $this->setUserId($data["user_id"]);
        $this->setProductId($data["product_id"]);        
    }

    public function toArray()
    {
        $data["id"] = $this->getId();
        $data["operation"] = $this->getType();
        $data["amount"] = $this->getAmount();
        $data["value"] = $this->getValue();
        $data["total"] = $this->getTotal();
        $data["user_id"] = $this->getUser();
        $data["product_id"] = $this->getProductId();

        return $data;
    }

}