<?php
namespace Entity;
use \DateTime;

/** @Entity
 *  @HasLifecycleCallbacks
 *  @Table(name="item")
 */
class Item
{

    /////////////////////////////////////////////////////////////////////////
    // Properties                                                          //
    /////////////////////////////////////////////////////////////////////////

    /**
    * @Id
    * @Column(type="integer")
    * @generatedValue(strategy="AUTO")
    */
    protected $id;
    /**
     * @Column(type="integer")
     */
    protected $amount=0;
    /**
     * @Column(type="integer")
     */
    protected $returned_amount=0;       
    /**
     * @Column(type="decimal", precision=12, scale=4)
     */
    protected $volume=0;
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $value=0;
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $sub_total=0;
    /**
     * @Column(type="decimal", precision=12, scale=4)
     */
    protected $width=0;
    /**
     * @Column(type="decimal", precision=12, scale=4)
     */
    protected $height=0;
    /**
     * @Column(type="decimal", precision=12, scale=3)
     */
    protected $thickness=0;    
    // Behaviors

    /**
     * @Column(type="datetime")
     */
    protected $created_at;
    /**
     * @Column(type="datetime")
     */
    protected $updated_at;
    /**
     * @Column(type="integer")
     * @version
     */
    protected $version;

    /////////////////////////////////////////////////////////////////////////
    // Relations                                                           //
    /////////////////////////////////////////////////////////////////////////

    /**
    * @ManyToOne(targetEntity="Sale", inversedBy="itens")
    * @JoinColumn(name="sale_id", referencedColumnName="id")
    */
    private $sale;    
    
    /**
    * @ManyToOne(targetEntity="Product", inversedBy="itens")
    * @JoinColumn(name="product_id", referencedColumnName="id")
    */
    private $product; 
    
    /////////////////////////////////////////////////////////////////////////
    // ForeingKey                                                           //
    /////////////////////////////////////////////////////////////////////////

    /**
     * @Column(type="integer")
     */
    protected $sale_id;
    /**
     * @Column(type="integer")
     */
    protected $product_id;    
    
    /////////////////////////////////////////////////////////////////////////
    // Getters/Setters                                                     //
    /////////////////////////////////////////////////////////////////////////

    public function getId() {return $this->id;}
    public function setAmount($value) {$this->amount = $value;}
    public function getAmount() {return $this->amount;}
    public function setReturnedAmount($value) {$this->returned_amount = $value;}
    public function getReturnedAmount() {return $this->returned_amount;}
    public function setVolume($value) {$this->volume = str_replace(',','.',str_replace('.','',$value));}
    public function getVolume() {return number_format($this->volume, 4, ',', '.');}
    public function setValue($value) {$this->value = str_replace(',','.',str_replace('.','',$value));}
    public function getValue() {return number_format($this->value, 2, ',', '.');}
    public function setSubTotal($value) {$this->sub_total = str_replace(',','.',str_replace('.','',$value));}
    public function getSubTotal() {return number_format($this->sub_total, 2, ',', '.');}
    public function setWidth($value) {$this->width = str_replace(',','.',str_replace('.','',$value));}
    public function getWidth() {return number_format($this->width, 4, ',', '.');}
    public function setHeight($value) {$this->height = str_replace(',','.',str_replace('.','',$value));}
    public function getHeight() {return number_format($this->height, 4, ',', '.');}
    public function setThickness($value) {$this->thickness = str_replace(',','.',str_replace('.','',$value));}
    public function getThickness() {return number_format($this->thickness, 4, ',', '.');}

    
    //Especiais
    public function addReturnedAmount($value) {$this->returned_amount += $value;}
    public function getVolumeEx() {return $this->volume;}
    public function getValueEx() {return $this->value;}
    
    public function setSale($value) {$this->sale = $value;}
    public function getSale() {return $this->sale;}
    public function setSaleId($value) {$this->sale_id = $value;}
    public function getSaleId() {return $this->sale_id;}
    
    public function setProduct($value) {$this->product = $value;}
    public function getProduct() {return $this->product;}
    public function setProductId($value) {$this->product_id = $value;}
    public function getProductId() {return $this->product_id;}
    
    public function getCreatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getUpdatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getVersion() {return $this->version;}

    /////////////////////////////////////////////////////////////////////////
    // Constructor                                                         //
    /////////////////////////////////////////////////////////////////////////
    
    public function __construct()
    {
        $this->created_at = $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Hooks                                                               //
    /////////////////////////////////////////////////////////////////////////

    /**
     * @PrePersist
     * @PreUpdate
     */
    public function Validate()
    {
        $this->ApplyBehaviors();

        $e = new ValidationException();

        if($e->hasError())
            throw $e;

    }

    private function ApplyBehaviors()
    {
        $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Helpers                                                             //
    /////////////////////////////////////////////////////////////////////////

    public function syncronize($data)
    {
        
        $this->setAmount($data["amount"]);
        $this->setReturnedAmount($data["returned_amount"]);
        $this->setVolume($data["volume"]);
        $this->setValue($data["value"]);
        $this->setSubTotal($data["sub_total"]);
        $this->setWidth($data["width"]);
        $this->setHeight($data["height"]);   
        $this->setThickness($data["thickness"]);   
        
        $this->setSaleId($data["sale_id"]);
        $this->setProductId($data["product_id"]);        
    }

    public function toArray()
    {
        $data["id"] = $this->getId();
        $data["amount"] = $this->getAmount();
        $data["returned_amount"] = $this->getReturnedAmount();
        $data["volume"] = $this->getVolume();
        $data["value"] = $this->getValue();
        $data["sub_total"] = $this->getSubTotal();
        $data["width"] = $this->getWidth();
        $data["height"] = $this->getHeight();
        $data["thickness"] = $this->getThickness();
        
        $data["sale_id"] = $this->getSale();
        $data["product_id"] = $this->getProductId();

        return $data;
    }

}