<?php
namespace Entity;
use \DateTime;

/** @Entity
 *  @HasLifecycleCallbacks
 *  @Table(name="payment")
 */
class Payment
{

    /////////////////////////////////////////////////////////////////////////
    // Properties                                                          //
    /////////////////////////////////////////////////////////////////////////

    /**
    * @Id
    * @Column(type="integer")
    * @generatedValue(strategy="AUTO")
    */
    protected $id;
    /**
     * @Column(type="integer")
     */
    protected $pay_mode;    
    /**
     * @Column(type="decimal", precision=12, scale=2)
     */
    protected $value=0;
    // Behaviors

    /**
     * @Column(type="datetime")
     */
    protected $created_at;
    /**
     * @Column(type="datetime")
     */
    protected $updated_at;
    /**
     * @Column(type="integer")
     * @version
     */
    protected $version;

    /////////////////////////////////////////////////////////////////////////
    // Relations                                                           //
    /////////////////////////////////////////////////////////////////////////

    /**
    * @ManyToOne(targetEntity="Sale", inversedBy="payments")
    * @JoinColumn(name="sale_id", referencedColumnName="id")
    */
    private $sale;    
    /////////////////////////////////////////////////////////////////////////
    // ForeingKey                                                           //
    /////////////////////////////////////////////////////////////////////////

    /**
     * @Column(type="integer")
     */
    protected $sale_id;
    
    /////////////////////////////////////////////////////////////////////////
    // Getters/Setters                                                     //
    /////////////////////////////////////////////////////////////////////////

    public function getId() {return $this->id;}
    public function setPayMode($value) {$this->pay_mode = $value;}
    public function getPayMode() {return $this->pay_mode;}    
    public function setValue($value) {$this->value = str_replace(',','.',str_replace('.','',$value));}
    public function getValue() {return number_format($this->value, 2, ',', '.');}
    
    public function setSale($value) {$this->sale = $value;}
    public function getSale() {return $this->sale;}
    public function setSaleId($value) {$this->sale_id = $value;}
    public function getSaleId() {return $this->sale_id;}
    
    public function getCreatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getUpdatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getVersion() {return $this->version;}

    /////////////////////////////////////////////////////////////////////////
    // Constructor                                                         //
    /////////////////////////////////////////////////////////////////////////
    
    public function __construct()
    {
        $this->created_at = $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Hooks                                                               //
    /////////////////////////////////////////////////////////////////////////

    /**
     * @PrePersist
     * @PreUpdate
     */
    public function Validate()
    {
        $this->ApplyBehaviors();

        $e = new ValidationException();

        if($e->hasError())
            throw $e;

    }

    private function ApplyBehaviors()
    {
        $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Helpers                                                             //
    /////////////////////////////////////////////////////////////////////////

    public function syncronize($data)
    {
        $this->setPayMode($data["pay_mode"]);
        $this->setValue($data["value"]);
        
        $this->setSaleId($data["sale_id"]);
    }

    public function toArray()
    {
        $data["id"] = $this->getId();
        $data["pay_mode"] = $this->getPayMode();
        $data["value"] = $this->getValue();
        
        $data["sale_id"] = $this->getSale();

        return $data;
    }

}