<?php
namespace Entity;
use \DateTime;

/** @Entity
 *  @HasLifecycleCallbacks
 *  @Table(name="user")
 */
class User
{

    /////////////////////////////////////////////////////////////////////////
    // Properties                                                          //
    /////////////////////////////////////////////////////////////////////////

    /**
    * @Id
    * @Column(type="integer")
    * @generatedValue(strategy="AUTO")
    */
    protected $id;
    /**
     * @Column(length=50)
     */
    protected $name;
    /**
     * @Column(length=100)
     */
    protected $email;
    /**
     * @Column(length=36)
     */
    protected $password;
    /**
     * @Column(type="text")
     */
    protected $permission;
    // Behaviors

    /**
     * @Column(type="datetime")
     */
    protected $created_at;
    /**
     * @Column(type="datetime")
     */
    protected $updated_at;
    /**
     * @Column(type="integer")
     * @version
     */
    protected $version;

    /////////////////////////////////////////////////////////////////////////
    // Relations                                                           //
    /////////////////////////////////////////////////////////////////////////
    
    /**
    * @OneToMany(targetEntity="History", mappedBy="user");
    */
    private $histories;
    
    /**
    * @OneToMany(targetEntity="Sale", mappedBy="user");
    */
    private $sales;
    
    /////////////////////////////////////////////////////////////////////////
    // Getters/Setters                                                     //
    /////////////////////////////////////////////////////////////////////////

    public function getId() {return $this->id;}
    public function setName($value) {$this->name = $value;}
    public function getName() {return $this->name;}
    public function setEmail($value) {$this->email = $value;}
    public function getEmail() {return $this->email;}
    public function setPassword($value) {$this->password = $value=="d41d8cd98f00b204e9800998ecf8427e"?"":$value;}
    public function getPassword() {return $this->password;}
    public function setPermission($value) {$this->permission = $value;}
    public function getPermission() {return $this->permission;}

    public function getCreatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getUpdatedAt() {return $this->created_at->format("d/m/Y H:i:s");}
    public function getVersion() {return $this->version;}

    /////////////////////////////////////////////////////////////////////////
    // Constructor                                                         //
    /////////////////////////////////////////////////////////////////////////
    
    public function __construct()
    {
        $this->created_at = $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Hooks                                                               //
    /////////////////////////////////////////////////////////////////////////

    /**
     * @PrePersist
     * @PreUpdate
     */
    public function Validate()
    {
        $this->ApplyBehaviors();

        $e = new ValidationException();

        if(empty($this->name))
        {
            $e->AppendError("O Campo nome não pode ser vazio.","name");
        }

        if(empty($this->email))
        {
            $e->AppendError("O Campo email não pode ser vazio.","email");
        }

        if(empty($this->password))
        {
            $e->AppendError("O Campo senha não pode ser vazio.","password");
        }

        if($e->hasError())
            throw $e;

    }

    private function ApplyBehaviors()
    {
        $this->updated_at = new DateTime("now");
    }

    /////////////////////////////////////////////////////////////////////////
    // Helpers                                                             //
    /////////////////////////////////////////////////////////////////////////

    public function syncronize($data)
    {
        $this->setName($data["name"]);
        $this->setEmail($data["email"]);
        $this->setPermission($data["permission"]);
    }

    public function toArray()
    {
        $data["id"] = $this->getId();
        $data["name"] = $this->getName();
        $data["email"] = $this->getEmail();
        $data["password"] = $this->getPassword();
        $data["permission"] = $this->getPermission();

        return $data;
    }

}