<?php

class Frontend_StockController extends Zend_Controller_Action
{

    public function  preDispatch()
    {
        $this->em = Zend_Registry::get("em");

        //desabilita o layout caso seja um XHR
        if ($this->_request->isXmlHttpRequest() || isset($_GET['ajax']))
            Zend_Controller_Action_HelperBroker::removeHelper('Layout');

        $this->ctrl = array (
        );
        
        $this->view->location = "stock";
    }

    public function operationAction()
    {
       BreadCrumb::clear();
       BreadCrumb::append("Estoque - Operação","/frontend/stock/operation");
    }
    
    public function indexAction()
    {
       BreadCrumb::clear();
       BreadCrumb::append("Estoque - Histórico","/frontend/stock");       
    }
    
    public function permissionDeniedAction()
    {
       BreadCrumb::clear();
       BreadCrumb::append("Home","/frontend/index/");
       BreadCrumb::append("Permissão negada","/frontend/index/permission-denied");
    }
    
    public function execAction()
    {
        $data = $this->_request->getParam("data");
        $data["user_id"] = Resource::getCurrentUserId(); 
        
        if($data["product_id"] == "")
            die("Produto não selecionado!");
        
        $amount = str_replace(',','.',str_replace('.','',$data["amount"]));

        if($amount == 0)
           die("A quantidade não pode ser zero!");
        
        switch($data["operation"])
        {
            case Resource::OPERATION_PURCHASE:
                Resource::stockPush($data["product_id"], $amount);
                break;
            case Resource::OPERATION_ENTRANCE_ADJUST:
                Resource::stockPush($data["product_id"], $amount);
                break;
            case Resource::OPERATION_EXIT_ADJUST:
                Resource::stockPop($data["product_id"], $amount);
                break;                
        }
        
        $h = new Entity\History();
        $h->syncronize($data); 
       
        $this->em->persist($h);
        $this->em->flush();
        $this->em->clear();

        
        die("ok");
    }
    
    public function listHistoryByProductAction()
    {
        $qb = $this->em->createQueryBuilder();
        $product_id = $this->_request->getParam("product_id");
        
        $qb->from("Entity\History", "h")
           ->InnerJoin("h.product","p")
           ->where("h.product_id={$product_id}");
               
        $select = "h.id,h.operation,p.code,p.name,p.code,h.amount,h.value,h.total,h.created_at";
        
        $filters = explode(",",$select);
       
        die(JQGrid::getJson($qb, $this->_request, $select, $filters, "h"));
    }
    
    public function listHistoryAction()
    {
        $qb = $this->em->createQueryBuilder();
        
        $date_start = $this->_request->getParam("date_start");
        $date_end = $this->_request->getParam("date_end");
        $export = $this->_request->getParam("export");
        
        $qb->from("Entity\History", "h")
           ->InnerJoin("h.product","p");
        
        if(!empty($date_start)) {
            $tmp = explode("/",$date_start);
            $date_start = $tmp[2] . "-" . $tmp[1] . "-" . $tmp[0] . " 00:00:00"; 
            $qb->andWhere("h.created_at >= '$date_start'");
        }
        
        if(!empty($date_end)) {
            $tmp = explode("/",$date_end);
            $date_end = $tmp[2] . "-" . $tmp[1] . "-" . $tmp[0] . " 23:59:59";
            
            if (!empty($date_start)) {
                $qb->andWhere("h.created_at >= '$date_start' AND h.created_at <= '$date_end'");
            } else {
                $qb->andWhere("h.created_at <= '$date_end'");
            }
        }        
               
        $select = "h.id,h.operation,p.code,p.name,p.code,h.amount,h.value,h.total,h.created_at";
        
        $filters = explode(",",$select);
        $qbWithoutOrder = clone $qb;
       
        $data = JQGrid::getJson($qb, $this->_request, $select, $filters, "h",$export);
        
        $data = json_decode($data,true);

        // $sum = JQGrid::getSum($qb, $this->_request,"sum(h.total) as sum_total,sum(h.amount) as sum_amount", $filters, "h");

        //Soma agrupado
        $qbWithoutOrder->groupBy("h.operation", 'h.created_at');
        
        $sum = JQGrid::getSum($qbWithoutOrder, $this->_request,"h.operation,sum(h.total) as sum_total,sum(h.amount) as sum_amount", $filters, "h");
        
        $data["userdata"]['amount'] = $sum[0]["sum_amount"];
        $data["userdata"]['total'] = $sum[0]["sum_total"];
        
        $groupedTotals = [];
        
        foreach ($sum as $group) {
            $total = !empty($groupedTotals[$group['operation']]) ? $groupedTotals[$group['operation']]['sum_total'] : 0;
            $amount = !empty($groupedTotals[$group['operation']]) ? $groupedTotals[$group['operation']]['sum_amount'] : 0;
            
            $groupedTotals[$group['operation']] = [
                'operation' => $group['operation'],
                'sum_total' => (float) $group['sum_total'] + $total,
                'sum_amount' => (float) $group['sum_amount'] + $amount
            ];
        }
        
        $data["userdata"]['grouped_totals'] = $groupedTotals;
        
        $data = json_encode($data);
        
        if($export)
        {
            $this->view->data = json_decode($data,true);
            $this->render("export");
        }
        
        die($data);
    }    

}
