<?php

class Frontend_ProductController extends Zend_Controller_Action {

    public function preDispatch()
    {
        $this->em = Zend_Registry::get("em");

        //desabilita o layout caso seja um XHR/Ajax
        if ($this->_request->isXmlHttpRequest() || isset($_GET['ajax']))
            Zend_Controller_Action_HelperBroker::removeHelper('Layout');

        $this->ctrl = array(
            'name' => "product",
            'class' => "Entity\Product"
        );
        
        $this->view->location = "product";
    }

    public function indexAction()
    {
        BreadCrumb::clear();
        BreadCrumb::append("Produtos","/frontend/product");
        
       if(!Permission::isChecked(Permission::VIEW_PRODUCT,Resource::getCurrentUserPermissionData()))
       {
           $this->_redirect("/frontend/index/permission-denied");
       }
        
    }

    public function listAction()
    {
        $qb = $this->em->createQueryBuilder();

        $qb->from($this->ctrl["class"], "p");

        $select = "p.id,p.name,p.code,p.measurement_unit,p.locale,p.value,p.thickness,p.amount";

        $filters = explode(",",$select);
        
        
        die(JQGrid::getJson($qb, $this->_request, $select, $filters, "p",false));
    }

    public function searchAction()
    {
        
        $term = $this->_request->getParam("term");
        $qb = $this->em->createQueryBuilder();
        
        $qb->select("p.id,p.name,p.code,p.value,p.amount,p.locale,p.thickness,p.measurement_unit")
           ->from($this->ctrl["class"], "p")
           ->where('p.status=1')
           ->setMaxResults(10);
                   
        $w1 = $qb->expr()->like("p.code","'%" . $term ."%'");
        $w2 = $qb->expr()->like("p.name","'%" . $term ."%'");
        
        $qb->andWhere($qb->expr()->orx($w1,$w2));    
        
        $result = $qb->getQuery()->getArrayResult();

        $data = array();
        
        foreach($result as $p)
        {
            $data[] = array(
                "id"=>$p["id"],
                "label"=>$p["code"] . " - " . $p["name"],
                "amount"=>number_format($p["amount"],4,",","."),
                "locale"=>Resource::$product_locale_list[$p["locale"]],
                "p_value"=>number_format($p["value"],2,",","."),
                "code"=>$p["code"],
                "name"=>$p["name"],
                "thickness"=>number_format($p["thickness"],3,",","."),
                "measurement_unit"=>$p["measurement_unit"]
                );
        }
        
        die(json_encode($data));
        
    }
    
    public function getInfoAction()
    {
        
        $id = $this->_request->getParam("id");
        $obj = $this->em->find($this->ctrl["class"], $id);
        
        $this->view->product = $obj->toArray();
    }
    
    public function addAction()
    {

        if(!Permission::isChecked(Permission::ADD_EDIT_PRODUCT,Resource::getCurrentUserPermissionData()))
        {
           $this->_redirect("/frontend/index/permission-denied");
        }
        
        BreadCrumb::clear();
        BreadCrumb::append("Produtos","/frontend/product");
        
        
        $data = $this->_request->getParam("data");
        $id = $data["id"];
        
        if(empty($id))
            BreadCrumb::append("Adicionar","/frontend/product/add");
        else
            BreadCrumb::append("Editar","/frontend/product/add");

        if ($this->_request->isPost()) {
            
            if (empty($id))
                $obj = new $this->ctrl["class"]();
            else
                $obj = $this->em->find($this->ctrl["class"], $id);

            $obj->syncronize($data);

            try {
                $this->em->persist($obj);
                $this->em->flush();
                $this->em->clear();
                $obj = $this->em->find($this->ctrl["class"], $obj->getId());
                $this->view->data = $obj->toArray();
                $this->view->show_success = true;
            } catch (\Entity\ValidationException $e) {
                $this->view->errors = $e->getMessages();
                $this->view->data = $data;
            } catch (Exception $e) {
                die("Erro Fatal: " . $e->getMessage());
            }
        } else {
            if (!is_null($id)) {
                $obj = $this->em->find($this->ctrl["class"], $id);
                $this->view->data = $obj->toArray();
            }
        }

        //Resources
    }

    public function deleteAction() {

        if(!Permission::isChecked(Permission::DELETE_PRODUCT,Resource::getCurrentUserPermissionData()))
        {
           die("permission-denied");
        }
        
        try
        {
            $id = $this->_request->getParam("id");
            $obj = $this->em->find($this->ctrl["class"], $id);

            $this->em->remove($obj);
            $this->em->flush();

            die("ok");
        }
        catch(Exception $e)
        {
            die("err");
        }
    }

}
